@@ -1,17 +1,16 @@
 from quick_orm.core import Database
 from sqlalchemy import Column, String
 
-__metaclass__ = Database.DefaultMeta
-
-class User:
+class User(object):
+    __metaclass__ = Database.DefaultMeta
     name = Column(String(70))
 
 if __name__ == '__main__':
     database = Database('sqlite://')
     database.create_tables()
     
-    user1 = User(name = 'Hello World')
-    database.session.add_then_commit(user1)
+    user = User(name = 'Hello World')
+    database.session.add_then_commit(user)
     
     user = database.session.query(User).get(1)
     print 'My name is', user.name
