@@ -1,16 +1,15 @@
 from quick_orm.core import Database
 from sqlalchemy import Column, String, Text
 
-__metaclass__ = Database.DefaultMeta
-
-@Database.many_to_many('role')
-class User:
+class User(object):
+    __metaclass__ = Database.DefaultMeta
     name = Column(String(70))
 
-class Role:
+@Database.many_to_many(User)
+class Role(object):
+    __metaclass__ = Database.DefaultMeta
     name = Column(String(70))
 
-
 if __name__ == '__main__':
     database = Database('sqlite://')
     database.create_tables()
