@@ -1,9 +1,8 @@
 from quick_orm.core import Database
 from sqlalchemy import Column, String
 
-__metaclass__ = Database.DefaultMeta
-
-class User:
+class User(object):
+    __metaclass__ = Database.DefaultMeta
     name = Column(String(70))
 
 if __name__ == '__main__':
