@@ -1,13 +1,13 @@
 from quick_orm.core import Database
 from sqlalchemy import Column, String, Text
 
-__metaclass__ = Database.DefaultMeta
-
-class User:
+class User(object):
+    __metaclass__ = Database.DefaultMeta
     name = Column(String(70))
 
 @Database.foreign_key(User)
-class Post:
+class Post(object):
+    __metaclass__ = Database.DefaultMeta
     content = Column(Text)
 
 class Question(Post):
@@ -21,7 +21,6 @@ class Answer(Post):
 class Comment(Post):
     pass
 
-
 if __name__ == '__main__':
     database = Database('sqlite://')
     database.create_tables()
